#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Aug  7 18:03:56 2019

@author: druee
"""

import unittest

import meteolib as m


class Test_magnus_w(unittest.TestCase):
    '''
    magnus_w(t, Kelvin=None, p=None, hPa=False)
    '''

    def test_2p_auto(self):
        """
        0°C -> 611.2 Pa
        20°C > 2336.1 Pa
        """
        self.assertAlmostEqual(m.humidity.magnus_w(0.), 611.2, places=1)
        self.assertAlmostEqual(m.humidity.magnus_w(20.), 2332.6, places=1)

    def test_2p_C(self):
        """
        0°C -> 611.2 Pa
        20°C > 2336.1 Pa
        """
        self.assertAlmostEqual(
            m.humidity.magnus_w(0., False), 611.2, places=1)
        self.assertAlmostEqual(m.humidity.magnus_w(
            20., False), 2332.6, places=1)

    def test_2p_K(self):
        """
        0°C -> 611.2 Pa
        20°C > 2336.1 Pa
        """
        self.assertAlmostEqual(m.humidity.magnus_w(
            273.15, True), 611.2, places=1)
        self.assertAlmostEqual(m.humidity.magnus_w(
            293.15, True), 2332.6, places=1)

    def test_2p_C_hPa(self):
        """
        0°C -> 6.112 hPa
        20°C > 23.361 hPa
        """
        self.assertAlmostEqual(m.humidity.magnus_w(
            0., False, hPa=True), 6.112, places=3)
        self.assertAlmostEqual(m.humidity.magnus_w(
            20., False, hPa=True), 23.326, places=3)

    def test_2p_C_Pa(self):
        """
        0°C -> 611.2 Pa
        20°C > 2336.1 Pa
        """
        self.assertAlmostEqual(m.humidity.magnus_w(
            0., False, hPa=False), 611.2, places=1)
        self.assertAlmostEqual(m.humidity.magnus_w(
            20., False, hPa=False), 2332.6, places=1)

    def test_2p_C_1013_hPa(self):
        """
        0°C -> 6.141 Pa *
        20°C > 23.436 Pa *
        """
        self.assertAlmostEqual(m.humidity.magnus_w(
            0., False, p=1013, hPa=True), 6.141, places=2)
        self.assertAlmostEqual(m.humidity.magnus_w(
            20., False, p=1013, hPa=True), 23.436, places=1)


class Test_magnus_i(unittest.TestCase):
    '''
    magnus_i(t, Kelvin=None, p=None, hPa=False)
    '''

    def test_2p_auto(self):
        """
        0°C-> 611.2 Pa
        -20°C > 103.3 Pa
        """
        self.assertAlmostEqual(m.humidity.magnus_i(0.), 611.2, places=1)
        self.assertAlmostEqual(m.humidity.magnus_i(-20.), 103.3, places=1)

    def test_2p_C(self):
        """
        0°C-> 611.2 Pa
        -20°C > 103.3 Pa
        """
        self.assertAlmostEqual(
            m.humidity.magnus_i(0., False), 611.2, places=1)
        self.assertAlmostEqual(
            m.humidity.magnus_i(-20., False), 103.3, places=1)

    def test_2p_K(self):
        """
        0°C-> 611.2 Pa
        -20°C > 103.3 Pa
        """
        self.assertAlmostEqual(m.humidity.magnus_i(
            273.15, True), 611.2, places=1)
        self.assertAlmostEqual(m.humidity.magnus_i(
            253.15, True), 103.3, places=1)

    def test_2p_C_hPa(self):
        """
        0°C-> 6.112 hPa
        -20°C > 1.033 hPa
        """
        self.assertAlmostEqual(m.humidity.magnus_i(
            0., False, hPa=True), 6.112, places=3)
        self.assertAlmostEqual(m.humidity.magnus_i(
            -20., False, hPa=True), 1.0326, places=3)

    def test_2p_C_Pa(self):
        """
        0°C-> 611.2 Pa
        -20°C > 103.3 Pa
        """
        self.assertAlmostEqual(m.humidity.magnus_i(
            0., False, hPa=False), 611.2, places=1)
        self.assertAlmostEqual(m.humidity.magnus_i(
            -20., False, hPa=False), 103.3, places=1)

    def test_2p_C_1013_hPa(self):
        """
        0°C-> 611.2 Pa *
        -20°C > 1.032 Pa *
        """
        self.assertAlmostEqual(m.humidity.magnus_i(
            0., False, p=1013, hPa=True), 6.112, places=1)
        self.assertAlmostEqual(m.humidity.magnus_i(
            -20., False, p=1013, hPa=True), 1.032, places=1)


class Test_tetens_w(unittest.TestCase):
    '''
    tetens_w(t, Kelvin=None, p=None, hPa=False)assertErrorEqual
    '''

    def test_2p_auto(self):
        self.assertAlmostEqual(m.humidity.tetens_w(0.), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.tetens_w(20.), 2338., delta=1.)

    def test_2p_C(self):
        self.assertAlmostEqual(
            m.humidity.tetens_w(0., False), 610.78, delta=1.)
        self.assertAlmostEqual(
            m.humidity.tetens_w(20., False), 2338., delta=1.)

    def test_2p_K(self):
        self.assertAlmostEqual(m.humidity.tetens_w(
            273.15, True), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.tetens_w(
            293.15, True), 2338., delta=1.)

    def test_2p_C_hPa(self):
        self.assertAlmostEqual(m.humidity.tetens_w(
            0., False, hPa=True), 6.1078, delta=0.01)
        self.assertAlmostEqual(m.humidity.tetens_w(
            20., False, hPa=True), 23.38, delta=0.01)

    def test_2p_C_Pa(self):
        self.assertAlmostEqual(m.humidity.tetens_w(
            0., False, hPa=False), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.tetens_w(
            20., False, hPa=False), 2338., delta=1.)

    def test_2p_C_1013_hPa(self):
        self.assertAlmostEqual(m.humidity.tetens_w(
            0., False, p=1013, hPa=True), 6.1180, delta=0.02)
        self.assertAlmostEqual(m.humidity.tetens_w(
            20., False, p=1013, hPa=True), 23.49, delta=0.02)


class Test_tetens_i(unittest.TestCase):
    '''
    tetens_i(t, Kelvin=None, p=None, hPa=False)
    '''

    def test_2p_auto(self):
        self.assertAlmostEqual(m.humidity.tetens_i(
            0.), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.tetens_i(
            -20.), 103., delta=1.)

    def test_2p_C(self):
        self.assertAlmostEqual(m.humidity.tetens_i(
            0., False), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.tetens_i(
            -20., False), 103., delta=1.)

    def test_2p_K(self):
        self.assertAlmostEqual(m.humidity.tetens_i(
            273.15, True), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.tetens_i(
            253.15, True), 103., delta=1.)

    def test_2p_C_hPa(self):
        self.assertAlmostEqual(m.humidity.tetens_i(
            0., False, hPa=True), 6.1078, delta=0.01)
        self.assertAlmostEqual(m.humidity.tetens_i(
            -20., False, hPa=True), 1.03, delta=0.01)

    def test_2p_C_Pa(self):
        self.assertAlmostEqual(m.humidity.tetens_i(
            0., False, hPa=False), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.tetens_i(
            -20., False, hPa=False), 103., delta=1.)

    def test_2p_C_1013_hPa(self):
        self.assertAlmostEqual(m.humidity.tetens_i(
            0., False, p=1013, hPa=True), 6.1180, delta=0.02)
        self.assertAlmostEqual(m.humidity.tetens_i(
            -20., False, p=1013, hPa=True), 1.03, delta=0.02)


class Test_goff_gratch_w(unittest.TestCase):
    '''
    goff_gratch_w(t, Kelvin=None, p=None, hPa=False)assertErrorEqual
    '''

    def test_2p_auto(self):
        self.assertAlmostEqual(m.humidity.goff_gratch_w(
            0.), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.goff_gratch_w(
            20.), 2338., delta=1.)

    def test_2p_C(self):
        self.assertAlmostEqual(m.humidity.goff_gratch_w(
            0., False), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.goff_gratch_w(
            20., False), 2338., delta=1.)

    def test_2p_K(self):
        self.assertAlmostEqual(m.humidity.goff_gratch_w(
            273.15, True), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.goff_gratch_w(
            293.15, True), 2338., delta=1.)

    def test_2p_C_hPa(self):
        self.assertAlmostEqual(m.humidity.goff_gratch_w(
            0., False, hPa=True), 6.1078, delta=0.01)
        self.assertAlmostEqual(m.humidity.goff_gratch_w(
            20., False, hPa=True), 23.38, delta=0.01)

    def test_2p_C_Pa(self):
        self.assertAlmostEqual(m.humidity.goff_gratch_w(
            0., False, hPa=False), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.goff_gratch_w(
            20., False, hPa=False), 2338., delta=1.)

    def test_2p_C_1013_hPa(self):
        self.assertAlmostEqual(m.humidity.goff_gratch_w(
            0., False, p=1013, hPa=True), 6.1180, delta=0.02)
        self.assertAlmostEqual(m.humidity.goff_gratch_w(
            20., False, p=1013, hPa=True), 23.49, delta=0.02)


class Test_goff_gratch_i(unittest.TestCase):
    '''
    goff_gratch_i(t, Kelvin=None, p=None, hPa=False)
    '''

    def test_2p_auto(self):
        self.assertAlmostEqual(m.humidity.goff_gratch_i(
            0.), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.goff_gratch_i(
            -20.), 103., delta=1.)

    def test_2p_C(self):
        self.assertAlmostEqual(m.humidity.goff_gratch_i(
            0., False), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.goff_gratch_i(
            -20., False), 103., delta=1.)

    def test_2p_K(self):
        self.assertAlmostEqual(m.humidity.goff_gratch_i(
            273.15, True), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.goff_gratch_i(
            253.15, True), 103., delta=1.)

    def test_2p_C_hPa(self):
        self.assertAlmostEqual(m.humidity.goff_gratch_i(
            0., False, hPa=True), 6.1078, delta=0.01)
        self.assertAlmostEqual(m.humidity.goff_gratch_i(
            -20., False, hPa=True), 1.03, delta=0.01)

    def test_2p_C_Pa(self):
        self.assertAlmostEqual(m.humidity.goff_gratch_i(
            0., False, hPa=False), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.goff_gratch_i(
            -20., False, hPa=False), 103., delta=1.)

    def test_2p_C_1013_hPa(self):
        self.assertAlmostEqual(m.humidity.goff_gratch_i(
            0., False, p=1013, hPa=True), 6.1180, delta=0.02)
        self.assertAlmostEqual(m.humidity.goff_gratch_i(
            -20., False, p=1013, hPa=True), 1.03, delta=0.02)


class Test_hyland_wexler_w(unittest.TestCase):
    '''
    hyland_wexler_w(t, Kelvin=None, p=None, hPa=False)assertErrorEqual
    '''

    def test_2p_auto(self):
        self.assertAlmostEqual(m.humidity.hyland_wexler_w(
            0.), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.hyland_wexler_w(
            20.), 2338., delta=1.)

    def test_2p_C(self):
        self.assertAlmostEqual(m.humidity.hyland_wexler_w(
            0., False), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.hyland_wexler_w(
            20., False), 2338., delta=1.)

    def test_2p_K(self):
        self.assertAlmostEqual(m.humidity.hyland_wexler_w(
            273.15, True), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.hyland_wexler_w(
            293.15, True), 2338., delta=1.)

    def test_2p_C_hPa(self):
        self.assertAlmostEqual(m.humidity.hyland_wexler_w(
            0., False, hPa=True), 6.1078, delta=0.01)
        self.assertAlmostEqual(m.humidity.hyland_wexler_w(
            20., False, hPa=True), 23.38, delta=0.01)

    def test_2p_C_Pa(self):
        self.assertAlmostEqual(m.humidity.hyland_wexler_w(
            0., False, hPa=False), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.hyland_wexler_w(
            20., False, hPa=False), 2338., delta=1.)

    def test_2p_C_1013_hPa(self):
        self.assertAlmostEqual(m.humidity.hyland_wexler_w(
            0., False, p=1013, hPa=True), 6.1180, delta=0.025)
        self.assertAlmostEqual(m.humidity.hyland_wexler_w(
            20., False, p=1013, hPa=True), 23.49, delta=0.025)


class Test_hyland_wexler_i(unittest.TestCase):
    '''
    hyland_wexler_i(t, Kelvin=None, p=None, hPa=False)
    '''

    def test_2p_auto(self):
        self.assertAlmostEqual(m.humidity.hyland_wexler_i(
            0.), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.hyland_wexler_i(
            -20.), 103., delta=1.)

    def test_2p_C(self):
        self.assertAlmostEqual(m.humidity.hyland_wexler_i(
            0., False), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.hyland_wexler_i(
            -20., False), 103., delta=1.)

    def test_2p_K(self):
        self.assertAlmostEqual(m.humidity.hyland_wexler_i(
            273.15, True), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.hyland_wexler_i(
            253.15, True), 103., delta=1.)

    def test_2p_C_hPa(self):
        self.assertAlmostEqual(m.humidity.hyland_wexler_i(
            0., False, hPa=True), 6.1078, delta=0.01)
        self.assertAlmostEqual(m.humidity.hyland_wexler_i(
            -20., False, hPa=True), 1.03, delta=0.01)

    def test_2p_C_Pa(self):
        self.assertAlmostEqual(m.humidity.hyland_wexler_i(
            0., False, hPa=False), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.hyland_wexler_i(
            -20., False, hPa=False), 103., delta=1.)

    def test_2p_C_1013_hPa(self):
        self.assertAlmostEqual(m.humidity.hyland_wexler_i(
            0., False, p=1013, hPa=True), 6.1180, delta=0.025)
        self.assertAlmostEqual(m.humidity.hyland_wexler_i(
            -20., False, p=1013, hPa=True), 1.03, delta=0.025)


class Test_sonntag_w(unittest.TestCase):
    '''
    sonntag_w(t, Kelvin=None, p=None, hPa=False)assertErrorEqual
    '''

    def test_2p_auto(self):
        self.assertAlmostEqual(m.humidity.sonntag_w(
            0.), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.sonntag_w(
            20.), 2338., delta=1.5)

    def test_2p_C(self):
        self.assertAlmostEqual(m.humidity.sonntag_w(
            0., False), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.sonntag_w(
            20., False), 2338., delta=1.5)

    def test_2p_K(self):
        self.assertAlmostEqual(m.humidity.sonntag_w(
            273.15, True), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.sonntag_w(
            293.15, True), 2338., delta=1.5)

    def test_2p_C_hPa(self):
        self.assertAlmostEqual(m.humidity.sonntag_w(
            0., False, hPa=True), 6.1078, delta=0.01)
        self.assertAlmostEqual(m.humidity.sonntag_w(
            20., False, hPa=True), 23.38, delta=0.015)

    def test_2p_C_Pa(self):
        self.assertAlmostEqual(m.humidity.sonntag_w(
            0., False, hPa=False), 610.78, delta=1.5)
        self.assertAlmostEqual(m.humidity.sonntag_w(
            20., False, hPa=False), 2338., delta=1.5)

    def test_2p_C_1013_hPa(self):
        self.assertAlmostEqual(m.humidity.sonntag_w(
            0., False, p=1013, hPa=True), 6.1180, delta=0.025)
        self.assertAlmostEqual(m.humidity.sonntag_w(
            20., False, p=1013, hPa=True), 23.49, delta=0.025)


class Test_sonntag_i(unittest.TestCase):
    '''
    sonntag_i(t, Kelvin=None, p=None, hPa=False)
    '''

    def test_2p_auto(self):
        self.assertAlmostEqual(m.humidity.sonntag_i(
            0.), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.sonntag_i(
            -20.), 103., delta=1.)

    def test_2p_C(self):
        self.assertAlmostEqual(
            m.humidity.sonntag_i(0., False), 610.78, delta=1.)
        self.assertAlmostEqual(
            m.humidity.sonntag_i(-20., False), 103., delta=1.)

    def test_2p_K(self):
        self.assertAlmostEqual(m.humidity.sonntag_i(
            273.15, True), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.sonntag_i(
            253.15, True), 103., delta=1.)

    def test_2p_C_hPa(self):
        self.assertAlmostEqual(m.humidity.sonntag_i(
            0., False, hPa=True), 6.1078, delta=0.01)
        self.assertAlmostEqual(m.humidity.sonntag_i(
            -20., False, hPa=True), 1.03, delta=0.01)

    def test_2p_C_Pa(self):
        self.assertAlmostEqual(m.humidity.sonntag_i(
            0., False, hPa=False), 610.78, delta=1.)
        self.assertAlmostEqual(m.humidity.sonntag_i(
            -20., False, hPa=False), 103., delta=1.)

    def test_2p_C_1013_hPa(self):
        self.assertAlmostEqual(m.humidity.sonntag_i(
            0., False, p=1013, hPa=True), 6.1180, delta=0.025)
        self.assertAlmostEqual(m.humidity.sonntag_i(
            -20., False, p=1013, hPa=True), 1.03, delta=0.025)


class Test_tdew(unittest.TestCase):
    '''
    tdew(e, p=None, hPa=False, Kelvin=True)
    '''

    def test_2p_auto(self):
        self.assertAlmostEqual(m.humidity.tdew(
            611.2), 273.15, places=1)
        self.assertAlmostEqual(m.humidity.tdew(
            2332.6), 293.15, places=1)

    def test_2p_C(self):
        self.assertAlmostEqual(m.humidity.tdew(
            611.2, Kelvin=False), 0.0, places=1)
        self.assertAlmostEqual(m.humidity.tdew(
            2332.6, Kelvin=False), 20.0, places=1)

    def test_2p_K(self):
        self.assertAlmostEqual(m.humidity.tdew(
            611.2, Kelvin=True), 273.15, places=1)
        self.assertAlmostEqual(m.humidity.tdew(
            2332.6, Kelvin=True), 293.15, places=1)

    def test_2p_C_hPa(self):
        self.assertAlmostEqual(m.humidity.tdew(
            6.1078, hPa=True, Kelvin=True), 273.15, places=1)
        self.assertAlmostEqual(m.humidity.tdew(
            23.326, hPa=True, Kelvin=True), 293.15, places=1)

    def test_2p_C_Pa(self):
        self.assertAlmostEqual(m.humidity.tdew(
            611.2, Kelvin=False, hPa=False), 0., places=1)
        self.assertAlmostEqual(m.humidity.tdew(
            2332.6, Kelvin=False, hPa=False), 20., places=1)

    def test_2p_C_1013_hPa(self):
        self.assertAlmostEqual(m.humidity.tdew(
            614.1, 101315., Kelvin=False, hPa=False), 0., places=1)
        self.assertAlmostEqual(m.humidity.tdew(
            2343.6, 101315., Kelvin=False, hPa=False), 20., places=1)


class Test_tfrost(unittest.TestCase):
    '''
    tfrost(e, p=None, hPa=False, Kelvin=True)
    '''

    def test_2p_auto(self):
        self.assertAlmostEqual(m.humidity.tfrost(
            611.2), 273.15, places=1)
        self.assertAlmostEqual(m.humidity.tfrost(
            103.3), 253.15, places=1)

    def test_2p_C(self):
        self.assertAlmostEqual(m.humidity.tfrost(
            611.2, Kelvin=False), 0.0, places=1)
        self.assertAlmostEqual(m.humidity.tfrost(
            103.3, Kelvin=False), -20.0, places=1)

    def test_2p_K(self):
        self.assertAlmostEqual(m.humidity.tfrost(
            611.2, Kelvin=True), 273.15, places=1)
        self.assertAlmostEqual(m.humidity.tfrost(
            103.3, Kelvin=True), 253.15, places=1)

    def test_2p_C_hPa(self):
        self.assertAlmostEqual(m.humidity.tfrost(
            6.1078, hPa=True, Kelvin=True), 273.15, places=1)
        self.assertAlmostEqual(m.humidity.tfrost(
            1.033, hPa=True, Kelvin=True), 253.15, places=1)

    def test_2p_C_Pa(self):
        self.assertAlmostEqual(m.humidity.tfrost(
            611.2, Kelvin=False, hPa=False), 0., places=1)
        self.assertAlmostEqual(m.humidity.tfrost(
            103.3, Kelvin=False, hPa=False), -20., places=1)

    def test_2p_C_1013_hPa(self):
        self.assertAlmostEqual(m.humidity.tfrost(
            614.1, 101315., Kelvin=False, hPa=False), 0., places=1)
        self.assertAlmostEqual(m.humidity.tfrost(
            103.3, 101315., Kelvin=False, hPa=False), -20., places=1)


class Test_psychro(unittest.TestCase):
    '''
    psychro(t, tw, Kelvin=None, p=None, hPa=False):
    '''

    def test_4p_auto(self):
        self.assertAlmostEqual(m.humidity.psychro(
            273.15, 273.15), 611.2, delta=1.)
        self.assertAlmostEqual(m.humidity.psychro(
            273.15, 272.15), 611.2*0.821, delta=1.)
        self.assertAlmostEqual(m.humidity.psychro(
            293.15, 293.15), 2332.6, delta=1.)
        self.assertAlmostEqual(m.humidity.psychro(
            293.15, 283.15), 2332.6*0.236, delta=10.)

    def test_4p_hPa(self):
        self.assertAlmostEqual(m.humidity.psychro(
            273.15, 273.15, hPa=True), 6.112, delta=0.01)
        self.assertAlmostEqual(m.humidity.psychro(
            273.15, 272.15, hPa=True), 6.112*0.821, delta=0.01)
        self.assertAlmostEqual(m.humidity.psychro(
            293.15, 293.15, hPa=True), 23.326, delta=0.01)
        self.assertAlmostEqual(m.humidity.psychro(
            293.15, 283.15, hPa=True), 23.326*0.236, delta=0.1)

    def test_4p_C(self):
        self.assertAlmostEqual(m.humidity.psychro(
            0., 0.), 611.2, delta=1.)
        self.assertAlmostEqual(m.humidity.psychro(
            0., -1.), 611.2*0.821, delta=1.)
        self.assertAlmostEqual(m.humidity.psychro(
            20., 20.), 2332.6, delta=1.)
        self.assertAlmostEqual(m.humidity.psychro(
            20., 10.), 2332.6*0.236, delta=10.)


class Test_psychro_ice(unittest.TestCase):
    '''
    psychro_ice(t, tw, Kelvin=None, p=None, hPa=False):
    '''

    def test_4p_auto(self):
        self.assertAlmostEqual(m.humidity.psychro_ice(
            273.15, 273.15), 611.2, delta=1.)
        self.assertAlmostEqual(m.humidity.psychro_ice(
            273.15, 272.15), 611.2*0.821, delta=10.)
        self.assertAlmostEqual(m.humidity.psychro_ice(
            253.15, 253.15), 103.26, delta=1.)
        self.assertAlmostEqual(m.humidity.psychro_ice(
            253.15, 252.15), 103.26*0.344, delta=10.)

    def test_4p_hPa(self):
        self.assertAlmostEqual(m.humidity.psychro_ice(
            273.15, 273.15, hPa=True), 6.112, delta=0.01)
        self.assertAlmostEqual(m.humidity.psychro_ice(
            273.15, 272.15, hPa=True), 6.112*0.821, delta=0.1)
        self.assertAlmostEqual(m.humidity.psychro_ice(
            253.15, 253.15, hPa=True), 1.0326, delta=0.01)
        self.assertAlmostEqual(m.humidity.psychro_ice(
            253.15, 252.15, hPa=True), 1.0326*0.344, delta=0.1)

    def test_4p_C(self):
        self.assertAlmostEqual(m.humidity.psychro_ice(
            0., 0.), 611.2, delta=1.)
        self.assertAlmostEqual(m.humidity.psychro_ice(
            0., -1.), 611.2*0.821, delta=10.)
        self.assertAlmostEqual(m.humidity.psychro_ice(
            -20., -20.), 103.26, delta=1.)
        self.assertAlmostEqual(m.humidity.psychro_ice(
            -20., -21.), 103.26*0.344, delta=10.)


class Test_relhum(unittest.TestCase):
    '''
    relhum(e, ew, percent=True)
    '''

    def test_3p_auto(self):
        self.assertAlmostEqual(m.humidity.relhum(
            0., 611.2), 0.0, delta=0.001)
        self.assertAlmostEqual(m.humidity.relhum(
            305.6, 611.2), 0.5, delta=0.001)
        self.assertAlmostEqual(m.humidity.relhum(
            611.2, 611.2), 1.0, delta=0.001)

    def test_3p_percent(self):
        self.assertAlmostEqual(m.humidity.relhum(
            0., 611.2, percent=True),  0.0, delta=0.1)
        self.assertAlmostEqual(m.humidity.relhum(
            305.6, 611.2, percent=True), 50.0, delta=0.1)
        self.assertAlmostEqual(m.humidity.relhum(
            611.2, 611.2, percent=True), 100.0, delta=0.1)


class Test_mixr(unittest.TestCase):
    '''
    mixr(e, p, gkg=False)
    '''

    def test_2p_auto(self):
        self.assertAlmostEqual(m.humidity.mixr(0., 1013.2), 0.0, delta=0.0001)
        self.assertAlmostEqual(m.humidity.mixr(
            6.112, 1013.2), 0.00375, delta=0.0001)

    def test_2p_hPa(self):
        self.assertAlmostEqual(m.humidity.mixr(
            611.2, 101320), 0.00375, delta=0.0001)

    def test_3p_gkg(self):
        self.assertAlmostEqual(m.humidity.mixr(
            0., 1013.2, gkg=True), 0.0, delta=0.1)
        self.assertAlmostEqual(m.humidity.mixr(
            6.112, 1013.2, gkg=True), 3.75, delta=0.1)
        self.assertAlmostEqual(m.humidity.mixr(
            611.2, 101320, gkg=True), 3.75, delta=0.1)


class Test_inv_mixr(unittest.TestCase):
    '''
    inv_mixr(m, p, hPa=False, gkg=False)
    '''

    def test_2p_auto(self):
        self.assertAlmostEqual(m.humidity.inv_mixr(
            0., 101320), 0.0, delta=1.)
        self.assertAlmostEqual(m.humidity.inv_mixr(
            0.00375, 101320), 611.2, delta=1.)

    def test_2p_hPa(self):
        self.assertAlmostEqual(m.humidity.inv_mixr(
            0., 1013.2, hPa=True), 0.0, delta=0.01)
        self.assertAlmostEqual(m.humidity.inv_mixr(
            0.00375, 1013.2, hPa=True), 6.112, delta=0.01)

    def test_3p_gkg(self):
        self.assertAlmostEqual(m.humidity.inv_mixr(
            0., 101320, gkg=True), 0.0, delta=1.)
        self.assertAlmostEqual(m.humidity.inv_mixr(
            3.75, 101320, gkg=True), 611.2, delta=1.)


class Test_spech(unittest.TestCase):
    '''
    spech(e, p, gkg=False)
    '''

    def test_2p_auto(self):
        self.assertAlmostEqual(m.humidity.spech(
            0., 1013.2), 0.0, delta=0.0001)
        self.assertAlmostEqual(m.humidity.spech(
            6.112, 1013.2), 0.00374, delta=0.0001)

    def test_2p_hPa(self):
        self.assertAlmostEqual(m.humidity.spech(
            611.2, 101320), 0.00374, delta=0.0001)

    def test_3p_gkg(self):
        self.assertAlmostEqual(m.humidity.spech(
            0., 1013.2, gkg=True), 0.0, delta=0.1)
        self.assertAlmostEqual(m.humidity.spech(
            6.112, 1013.2, gkg=True), 3.74, delta=0.1)
        self.assertAlmostEqual(m.humidity.spech(
            611.2, 101320, gkg=True), 3.74, delta=0.1)


class Test_inv_spech(unittest.TestCase):
    '''
    inv_spech(m, p, hPa=False, gkg=False)
    '''

    def test_2p_auto(self):
        self.assertAlmostEqual(
            m.humidity.inv_spech(0., 101320), 0.0, delta=1.)
        self.assertAlmostEqual(m.humidity.inv_spech(
            0.00374, 101320), 611.2, delta=1.)

    def test_2p_hPa(self):
        self.assertAlmostEqual(m.humidity.inv_spech(
            0., 1013.2, hPa=True), 0.0, delta=0.01)
        self.assertAlmostEqual(m.humidity.inv_spech(
            0.00374, 1013.2, hPa=True), 6.112, delta=0.01)

    def test_3p_gkg(self):
        self.assertAlmostEqual(m.humidity.inv_spech(
            0., 101320, gkg=True), 0.0, delta=1.)
        self.assertAlmostEqual(m.humidity.inv_spech(
            3.74, 101320, gkg=True), 611.2, delta=1.)


class Test_to_rh(unittest.TestCase):
    '''
    Humidity(t, p,
             e=None, rh=None, td=None, tf=None,
             tw=None, ti=None, q=None, m=None,
             Kelvin=None, hPa=False, gkg=False, percent=False):
    '''
    # test input variants

    def test_rh_t_auto(self):
        self.assertAlmostEqual(m.humidity.Humidity(
            rh=0.50, t=0.).e, 305.6, delta=1.)

    def test_e_t_auto(self):
        self.assertAlmostEqual(m.humidity.Humidity(
            e=611.2, t=0.).e, 611.2, delta=1.)

    def test_e_ew_auto(self):
        self.assertAlmostEqual(m.humidity.Humidity(
            e=611.2, ew=611.2).e, 611.2, delta=1.)

    def test_e_w_auto(self):
        self.assertAlmostEqual(m.humidity.Humidity(
            e=611.2, ew=611.2).e, 611.2, delta=1.)

    def test_e_t_percent(self):
        self.assertAlmostEqual(m.humidity.Humidity(
            e=611.2, t=0., percent=True).e, 611.2, delta=1.)

    def test_e_t_hPa(self):
        self.assertAlmostEqual(m.humidity.Humidity(
            e=6.112, t=0., hPa=True).e, 6.112, delta=0.01)

    def test_e_t_Kelvin(self):
        self.assertAlmostEqual(m.humidity.Humidity(
            e=611.2, t=273.15, Kelvin=True).e, 611.2, delta=1.)

    def test_td_t_auto(self):
        self.assertAlmostEqual(m.humidity.Humidity(
            td=9.327, t=20.).e, 1172., delta=1.)

    def test_tf_t_auto(self):
        self.assertAlmostEqual(m.humidity.Humidity(
            tf=8.13, t=20.).e, 1172., delta=1.)

    def test_tw_t_auto(self):
        self.assertAlmostEqual(m.humidity.Humidity(
            tw=13.87, t=20.).e, 1172., delta=1.)

    def test_ti_t_auto(self):
        self.assertAlmostEqual(m.humidity.Humidity(
            ti=-12, t=-10.).e, 100.8, delta=1.)

    def test_q_t_auto(self):
        self.assertAlmostEqual(m.humidity.Humidity(
            q=0.00714, p=101325, t=20.).e, 1172., delta=1.)

    def test_m_t_auto(self):
        self.assertAlmostEqual(m.humidity.Humidity(
            m=0.00719, p=101325, t=20.).e, 1172., delta=1.)
    # test execptions

    def test_params_notew(self):
        self.assertRaises(ValueError, m.humidity.Humidity, e=611.2)

    def test_params_tandew(self):
        self.assertRaises(ValueError, m.humidity.Humidity,
                          e=611.2, t=0., ew=611.2)

    def test_params_none(self):
        self.assertRaises(TypeError, m.humidity.Humidity,
                          e=611.2, t=0., Kelvin='blah')
        self.assertRaises(ValueError, m.humidity.Humidity,
                          e=611.2, t=0., hPa=None)
        self.assertRaises(ValueError, m.humidity.Humidity,
                          e=611.2, t=0., gkg=None)
        self.assertRaises(ValueError, m.humidity.Humidity,
                          e=611.2, t=0., percent=None)

    def test_rh_4p_auto(self):
        self.assertAlmostEqual(m.humidity.Humidity(
            rh=1., t=0., p=101320).rh(), 1., delta=0.01)
        self.assertAlmostEqual(m.humidity.Humidity(
            rh=0.5, t=0., p=101320).rh(), 0.5, delta=0.01)
        self.assertAlmostEqual(m.humidity.Humidity(
            rh=50., t=0., percent=True).rh(), 50., delta=1.)
        self.assertAlmostEqual(m.humidity.Humidity(
            rh=0.5, t=20., p=101320).rh(), 0.5, delta=0.01)

    def test_m_auto(self):
        self.assertAlmostEqual(m.humidity.Humidity(
            e=305.6, t=273.15, p=101320).m(), 0.00187, delta=0.0001)

    def test_m_gkg(self):
        self.assertAlmostEqual(m.humidity.Humidity(
            e=305.6, t=273.15, p=101320, gkg=True).m(), 1.87, delta=0.1)

    def test_q_auto(self):
        self.assertAlmostEqual(m.humidity.Humidity(
            e=305.6, t=273.15, p=101320).q(), 0.00187, delta=0.0001)

    def test_q_gkg(self):
        self.assertAlmostEqual(m.humidity.Humidity(
            e=305.6, t=273.15, p=101320, gkg=True).q(), 1.87, delta=0.1)

    def test_tw_4p_auto(self):
        self.assertAlmostEqual(m.humidity.Humidity(
            e=611.2, t=273.15).tw(), 273.15, delta=0.1)
        self.assertAlmostEqual(m.humidity.Humidity(
            e=611.2*0.821, t=273.15).tw(), 272.15, delta=0.1)
        self.assertAlmostEqual(m.humidity.Humidity(
            e=2332.6, t=293.15).tw(), 293.15, delta=0.1)
        self.assertAlmostEqual(m.humidity.Humidity(
            e=2332.6*0.236, t=293.15).tw(), 283.15, delta=0.1)

    def test_ti_4p_auto(self):
        self.assertAlmostEqual(m.humidity.Humidity(
            e=611.2, t=273.15).ti(), 273.15, delta=0.1)
        self.assertAlmostEqual(m.humidity.Humidity(
            e=611.2*0.821, t=273.15).ti(), 272.15, delta=0.1)
        self.assertAlmostEqual(m.humidity.Humidity(
            e=103.26, t=253.15).ti(), 253.15, delta=0.1)
        self.assertAlmostEqual(m.humidity.Humidity(
            e=103.26*0.344, t=253.15).ti(), 252.15, delta=0.1)

    def test_virt_inc_4p_auto(self):
        self.assertAlmostEqual(m.humidity.Humidity(
            t=0., td=0., p=101320).virt_inc(), 0.63, delta=0.01)
        self.assertAlmostEqual(m.humidity.Humidity(
            t=0., td=-1., p=101320).virt_inc(), 0.58, delta=0.01)
        self.assertAlmostEqual(m.humidity.Humidity(
            t=20., td=10., p=101320).virt_inc(), 1.35, delta=0.01)
        self.assertAlmostEqual(m.humidity.Humidity(
            t=20., td=20., p=101320).virt_inc(), 2.58, delta=0.05)

    def test_Tvirt_4p_auto(self):
        self.assertAlmostEqual(m.humidity.Humidity(
            t=0., td=0., p=101320).tvirt(), 273.15+0.63, delta=0.01)
        self.assertAlmostEqual(m.humidity.Humidity(
            t=0., td=-1., p=101320).tvirt(), 273.15+0.58, delta=0.01)
        self.assertAlmostEqual(m.humidity.Humidity(
            t=20., td=10., p=101320).tvirt(), 273.15+21.35, delta=0.01)
        self.assertAlmostEqual(m.humidity.Humidity(
            t=20., td=20., p=101320).tvirt(), 273.15+22.58, delta=0.05)

    def test_rhow_4p_auto(self):
        self.assertAlmostEqual(m.humidity.Humidity(
            rhow=0.0013, t=0.).rhow(), 0.0013, delta=0.0001)
        self.assertAlmostEqual(m.humidity.Humidity(
            rhow=0.0013, t=20.).rhow(), 0.0013, delta=0.0001)
        self.assertAlmostEqual(m.humidity.Humidity(
            rhow=0.013, t=0.).rhow(), 0.013, delta=0.0001)
        self.assertAlmostEqual(m.humidity.Humidity(
            rhow=0.013, t=20.).rhow(), 0.013, delta=0.0001)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
'''
Funtions representing the 1975 ISO Standard Atmosphere [ISO2533]_,
wich is identical to the ICAO standard atmosphere [ICAO7488]_
'''
import numpy as np
import pandas as pd

# from ._utils import _check, _only
from .constants import R, r_earth
from .pressure import _to_Pa
from .temperature import _to_K, _to_C
from .thermodyn import gas_rho

# Table 4 from [ISO2533]_ defining the temperature profile
# with respect to geopotential height
_tab4 = pd.DataFrame.from_records([
    [-2000, 301.15, -6.5E-3],
    [0, 288.15, -6.5E-3],
    [11000, 216.65,  0.0E0],
    [20000, 216.65,  1.0E-3],
    [32000, 228.65,  2.8E-3],
    [47000, 270.65,  0.0E0],
    [51000, 270.65, -2.8E-3],
    [71000, 214.65, -2.0E-3],
    [80000, 196.65, 0.0E0]],
    columns=["H_b", "T_b", "beta"])

# Table 5 from [ISO2533]_ giving tabbed values
# with respect to geopotential height
_tab5 = pd.DataFrame.from_records([
    [-2000, 127783., 1.47816E-0],
    [0, 101325., 1.22500E-0],
    [11000, 22632.0, 3.63918E-1],
    [20000, 5474.87, 8.80345E-2],
    [32000, 868.014, 1.35551E-2],
    [47000, 110.906, 1.42752E-3],
    [51000, 66.9384, 8.61600E-4],
    [71000, 3.95639, 6.42105E-5],
    [80000, 0.886272, 1.57004E-5]],
    columns=["H", "p", "rho"])

pzero_iso = 101325.  # Pa
R_iso = 287.05287    # K

# ---------------------------------------------------------------------


def beta_iso(H):
    if H < _tab4["H_b"][0]:
        res = None
    else:
        for i in _tab4.index:
            if H <= _tab4["H_b"][i]:
                res = _tab4["beta"][i]
                break
    return res


# ----------------------------------------------------------------------
def gphi(lat=None, h=0, deg=True):
    r"""
    Lambert's function for acceleration of gravity as function of latitude.
    Standard ("normal") gravity conforms with latitude
    :math:`\phi = 45^\circ32'33"`.
    For height dependence, centrifugal acceleration is
    formally neglected and using only Newton’s gravitation law.
    Defined in ICAO/ANSI/ISO standard atmosphere [ISO2533]_

    :param lat: (float, optional) latitude in radians
        (or degrees, if `deg` is True),
        positive to the north. If missing, standard gravity is returned.
    :param deg: (float, altitude) is geometric height.
    :param deg: (bool, optional) if True `lat` is in degrees,
        if False `lat` is in radians. Defaults to True (i.e. degrees).

    :returns: gravity im :math:`m/s^{-2}` (float)
    :rtype: float
    """
    if lat is None:
        lat = 45. + 32./60. + 33./3600.
        deg = True
    if deg:
        phi = np.deg2rad(lat)
    else:
        phi = lat
    # ISO 2533 Sect. 2.1 (unnumbered eqn)
    res = 9.80616 * (1 -
                     0.0026373 * np.cos(2 * phi) +
                     0.0000059 * np.square(np.cos(2 * phi)))

    res = res * (r_earth/(r_earth + h)) ** 2
    return res

# ---------------------------------------------------------------------


def T_iso(h, gpm=False, Kelvin=True):
    """
    Temperature in the standard atmosphere at given height

    :param h: (float) height above sea level (altitude) in m
    :param gpm: (bool, optional) if True, h is given in gepotential meters,
        if False h is given in gemetric height. Default to False.
    :param Kelvin: (bool, optional) if True temperature is returned in Kelvin,
        if False, temperature is returned in degree Celsius.
        Defaults to True.
    :returns: Temperature in Kelvin or degrees Celsius,
        depending on `Kelvin`
    :rtype: float
    """
    if not gpm:
        alt = h_geopot(h)
    else:
        alt = h
    for i in reversed(_tab4.index):
        if alt >= _tab4["H_b"][i]:
            T = (_tab4["T_b"][i] +
                 (alt - _tab4["H_b"][i]) * _tab4["beta"][i])
            break
    if Kelvin is True:
        return T
    else:
        return _to_C(T, Kelvin=True)

# ---------------------------------------------------------------------


def h_geopot(h_geomet):
    """
    converts geometric altitude to geopotential altitude
    :param h_geomet: (float) geometric altitude in m

    :returns: geopotential altitude in gpm
    :rtype: float
    """
    # ISO 2533 eqn 8
    return (r_earth * h_geomet) / (r_earth + h_geomet)

# ---------------------------------------------------------------------


def h_geomet(h_geopot):
    """
    converts geometric altitude to geopotential altitude
    :param h_geopot: (float) geopotential altitude in gpm

    :returns: geometric altitude in m
    :rtype: float
    """
    # ISO 2533 eqn 9
    return (r_earth * h_geopot) / (r_earth - h_geopot)

# ---------------------------------------------------------------------


def p_barom(h, p_0=pzero_iso, T_0=_tab4["T_b"][1], h_0=0,
            beta=_tab4["beta"][1],
            gpm=False, Kelvin=None, hPa=None):
    r"""
    barometric height formula

    :param h: (float) height above sea level (altitude) (:math:`m`)
    :param p_0: (float, optional) air pressure at base level
      (in hPa or Pa, depending on `hPa`).
      Defaults to `pzero_iso`
    :param T_0: (float, optional) air temperature at base level
      in :math:`K` or :math:`^\circ C` , depending on `Kelvin` .
      Defaults to 288.15 K.
    :param h_0: (float, optional) height of the base level
      above sea level (altitude of the base level) (:math:`m`).
      Defaults to 0 m.
    :param beta: (float, optional) vertical temperature gradient
      (:math:`K m^{-1}`). Defaults to 0.0065 :math:`K m^{-1}`.
    :param gpm: (bool, optional) if True, h is given in gepotential meters,
      if False h is given in gemetric height. Default to False.
    :param Kelvin: (optional, optional)
      if ``False``, all temperatures are assumed to be Kelvin.
      If ``False``, all temperatures are assumed to be Celsius.
      If missing of ``None``, unit  temperatures are autodetected.
      Defaults to ``None``.
    :param hPa: (optional, optional)
      if ``True``,  `p`, `e`, or `ew` must be supplied in hPa.
      If ``False``, `p`, `e`, or `ew` must be supplied in Pa.
      Defaults to ``False``.

    :return: air pressure (in hPa or Pa, depending on `hPa`).
    :rtype: float

    """
    Tnull = _to_K(T_0, Kelvin)  # K
    pnull = _to_Pa(p_0, hPa)    # Pa
    if not gpm:
        hnull = h_geopot(h_0)   # gpm
        alt = h_geopot(h)       # gpm
    else:
        hnull = h_0             # gpm
        alt = h                 # gpm
    gamma = beta                # K/m

    if abs(gamma) > 1.E-6:  # i.e. is not zero
        # ISO 2533 eqn 12
        pwr = - gphi() / (R_iso * gamma)
        pp = p_0 * (1. + (gamma * (alt - hnull) / Tnull)) ** pwr  # Pa
    else:
        # ISO 2533 eqn 13
        pp = p_0 * np.exp(- (gphi() / (R * Tnull)) * (alt - hnull))  # Pa

    if hPa is None:
        # convert to the same unit as supplied p_0 (Pa if p_0 not given)
        p = pp * p_0 / pnull
    elif hPa is True:
        p = pp / 100.
    else:
        p = pp
    return p

# ---------------------------------------------------------------------


def p_iso(h, p_0=None, gpm=False, hPa=False):
    """
    air pressure in the standard atmosphere at given height

    :param h: (float) height above sea level (altitude) (:math:`m`)
    :param p_0: (float, optional) air pressure at sea level
        (in hPa or Pa, depending on `hPa`).
        Defaults to `pzero_iso`
    :param gpm: (bool, optional) if True, h is given in gepotential meters,
        if False h is given in gemetric height. Default to False.
    :param hPa: (optional, optional)
      if ``True``,  `p` must be supplied in hPa.
      If ``False``, `p` must be supplied in Pa.
      Defaults to ``False``.

    :return: air pressure (in hPa or Pa, depending on `hPa`).
    :rtype: float
    """
    if p_0 is None:
        pp_0 = pzero_iso             # Pa
    else:
        pp_0 = _to_Pa(p_0, hPa=hPa)  # Pa
    # geopotential height
    if not gpm:
        alt = h_geopot(h)  # gpm
    else:
        alt = h            # gpm
    if alt < _tab4["H_b"][0]:
        integr = np.nan
    elif _tab4["H_b"][0] < alt < 0.:
        integr = p_barom(alt, p_0=pp_0, gpm=True, hPa=False)  # Pa
    else:
        pfact = pp_0 / pzero_iso
        for i in reversed(_tab4.index[1:]):
            if alt >= _tab4["H_b"][i]:
                integr = p_barom(h=alt,
                                 p_0=_tab5["p"][i] * pfact,
                                 h_0=_tab4["H_b"][i],
                                 T_0=_tab4["T_b"][i],
                                 beta=_tab4["beta"][i],
                                 gpm=True,
                                 Kelvin=True,
                                 hPa=False,
                                 )  # Pa
                break
        else:
            return np.nan
    if hPa is True:
        p = integr / 100.
    else:
        p = integr
    return p

# ---------------------------------------------------------------------


def rho_iso(h, p_0=pzero_iso, gpm=False):
    """
    density of dry air in the standard atmosphere at given height

    :param h: altitude (float) in m or gpm, depending on `gpm`
    :param p_0: (float, optional) air pressure at base level
        (in hPa or Pa, depending on `hPa`).
        Defaults to `pzero_iso`
    :param gpm: (bool, optional)
        if ``True``, h is given in gepotential meters,
        if ``False`` h is given in gemetric height. Default to ``False``.

    :return: density in :math:`kg m^{-3}`
    """
    return gas_rho(p_iso(h, p_0=p_0, gpm=gpm, hPa=False),
                   T_iso(h, gpm=gpm, Kelvin=True),
                   q=0.,
                   Kelvin=True,
                   hPa=False)

# ---------------------------------------------------------------------


def altitude(p, p_0=None, gpm=False, hPa=False):
    """
    returns altitude (height above se level) as funtion of pressure

    :param p: (float) air pressure (in hPa or Pa, depending on `hPa`).
    :param p_0: (float, optional) air pressure at base level
        (in hPa or Pa, depending on `hPa`).
        Defaults to `pzero_iso`
    :param gpm: (bool, optional) if ``True``,
        h is returned in gepotential meters,
        if ``False`` h is returned in gemetric height. Default to False.
    :param hPa: (optional, optional)
      if ``True``,  `p` must be supplied in hPa.
      If ``False``, `p` must be supplied in Pa.
      Defaults to ``False``.
    :return: altitude (float) in m or gpm, depending on `gpm`
    """
    if p_0 is None:
        pp_0 = pzero_iso             # Pa
    else:
        pp_0 = _to_Pa(p_0, hPa=hPa)  # Pa
    pp = _to_Pa(p, hPa=hPa)  # Pa
    if pp > _tab5["p"][0]:
        H = np.nan
    else:
        for i in reversed(_tab5.index):
            pi = _tab5["p"][i] * pp_0 / pzero_iso  # Pa
            if pp <= pi:
                beta = _tab4["beta"][i]  # K/m
                p_0 = pi      # Pa
                H_0 = _tab4["H_b"][i]    # gpm
                T_0 = _tab4["T_b"][i]    # K
                if abs(beta) > 1.E-6:
                    pwr = - gphi() / (R_iso * beta)
                    H = (((pp / p_0) ** (1/pwr) - 1) * (T_0 / beta) +
                         H_0)  # gpm
                else:
                    H = (-np.log(pp / p_0) * R_iso * T_0 / gphi() +
                         H_0)  # gpm
                break
    if gpm:
        res = H  # gpm
    else:
        res = h_geomet(H)  # m
    return res


if __name__ == "__main__":
    print(p_iso(0))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
provides meteorogical charts
"""
import logging
import os

import meteolib.humidity
import meteolib.standard
import numpy as np
import pandas as pd

try:
    import matplotlib
    if os.name == 'posix' and "DISPLAY" not in os.environ:
        matplotlib.use('Agg')
        have_display = False
    else:
        have_display = True
    from matplotlib import pyplot as plt
    have_matplotlib = True
except ImportError:
    matplotlib = None
    plt = None
    have_matplotlib = False
    have_display = False

from .constants import R, Lv, cp
from .humidity import Humidity
from .standard import altitude, p_iso, T_iso
from .temperature import KtoC, CtoK, inv_Tpot
from .temperature import POTENTIAL_REFERENCE_PRESSURE as PREF

logger = logging.getLogger()


# ----------------------------------------------------------------------
# adiabatic process lines
# ----------------------------------------------------------------------

def dryad(t0, pmax, pmin):
    """
    Calulates the temperature of an air parcel lifted
    adiabaticially from pmax to pmin,
    i.e. a (dry) adiabate.
    The parcel is assumed to attain the (potential)
    temperature t0 at reference level (generally 1000 hPa).

    :param t0: label temperature in C
    :type t0: float
    :param pmax: upper pressure limit in hPa (lifting start level)
    :type pmax: float
    :param pmin: lower pressure limit in hPa (lifting end level)
    :type pmin: float
    :return: pressure (in hPa) and temperature (in C) of air parcel
    :rtype: list[float], list[float]
    """
    p = np.arange(pmax, pmin, - 1)
    t = [inv_Tpot(t0, x, hPa=True, Kelvin=False) for x in p]
    return p, t


def satad(th, pmax, pmin, label=None):
    r"""
    Calulates the temperature of an air parcel lifted
    (pseudo)-adiabaticially from pmax to pmin,
    assuming all humidity condensating is immedately removed,
    i.e. a moist adiabate.
    The parcel is assumed to either attain the equivalent potential
    temperature th at great height, if label is `the` or to
    attain wet-bulb potential temperature th if label is `thw`.

    To calbulate wet-bulb potential temperature (:math:`\theta_w`) from
    equivalent potential temperature (:math:`\theta_e`) equation (3.8)
    from [DavJo2008]_.
    The vertical integration follows the process described by
    equation (8) in [BaSt2913]_.

    :param th: label temperature in C
    :type th: float
    :param pmax: upper pressure limit in hPa (lifting start level)
    :type pmax: float
    :param pmin: lower pressure limit in hPa (lifting end level)
    :type pmin: float
    :param label: type of label of air parcel. Possible are
        `the` for equivalent potential temperature (:math:`\theta_e`) or
        `thw` for wet-bulb potential temperature (:math:`\theta_w`).
    :return: pressure (in hPa) and temperature (in C) of air parcel
    :rtype: list[float], list[float]
    """
    the = None
    thw = None

    if label is None:
        logging.warning('No label for satad. Assuming "the".')
        label = "the"
    if label == "thw":
        thw = th
    elif label == "the":
        the = th

    if thw is None:
        c = 273.15
        thek = CtoK(the)
        x = thek/c
        if thek >= 173.15:  # K
            a0 = 7.101574
            a1 = -20.68208
            a2 = 16.11182
            a3 = 2.574631
            a4 = -5.205688
            b1 = -3.552497
            b2 = 3.781782
            b3 = -0.6899655
            b4 = -0.5929340
            thw = (thek - c -
                   np.exp((a0 + a1 * x + a2 * x * x +
                           a3 * x ** 3 + a4 * x ** 4) /
                          (1 + b1 * x + b2 * x * x +
                           b3 * x ** 3 + b4 * x ** 4))
                   )
        else:
            thw = thek - c  # C
        thw = CtoK(thw)  # K
    else:
        thw = CtoK(thw)  # K

    p0 = PREF / 100.  # hPa
    dp = -1  # hPa
    pp = np.arange(p0, pmin, dp)  # hPa

    pout = []
    tout = []
    tk = thw
    for p in pp:  # hPa
        epsilon = 0.622
        b = 1.
        m = Humidity(t=tk, Kelvin=True, p=p, hPa=True,
                     rh=1., percent=False).m(gkg=False)  # 1
        dtdp = (b / (p * 100.)) * \
               (R * tk + Lv * m) / (
                       cp + (Lv * Lv * m * epsilon * b) / (R * tk * tk)
               )  # K/Pa
        tk = tk + dtdp * (dp * 100.)  # K (dp: Pa -> hPa)
        tc = KtoC(tk)  # C
        if pmax >= p >= pmin:  # hPa
            pout.append(p)  # hPa
            tout.append(tc)  # C
    return pout, tout


def tdm(m, pmax, pmin):
    """
    Calulates the dew point of an air parcel lifted from
    reference level (generally 1000 hPa) maintaining
    a constant mixing ratio `m`

    :param m: mixing ratio (unitless, i.e. 1. = saturation)
    :type m: float
    :param pmax: upper pressure limit in hPa (lifting start level)
    :type pmax: float
    :param pmin: lower pressure limit in hPa (lifting end level)
    :type pmin: float
    :return: pressure (in hPa) and temperature (in C) of air parcel
    :rtype: list[float], list[float]
    """

    p = np.arange(pmax, pmin, -1)
    pout = []
    tout = []
    for pp in p:
        tt = T_iso(altitude(pp, hPa=True), Kelvin=False)
        td = meteolib.humidity.Humidity(m=m, t=tt, gkg=True, p=pp,
                                        Kelvin=False, hPa=True).td()
        if pmax >= pp >= pmin:
            pout.append(pp)
            tout.append(td)
    return pout, tout


# ----------------------------------------------------------------------
# Stüve Diagram
# ----------------------------------------------------------------------


def stueve(p=None, t=None, td=None, style=None, title=None,
           fname=None, fmt=None, dpi=300):
    """
    Produces a Stüve plot in the given style.
    Optionally, a vertical sounding may be supplied by giving
    pressure `p`, temperature `t` and/or dew point `td` as
    lists of equal length.

    :param p: optional, sounding pressure levels (in hPa)
    :type p: list[float]
    :param t: optional, sounding temperature values (in C)
    :type t: list[float]
    :param td: optional, sounding dew point temperatures (in C)
    :type td: list[float]
    :param style: optional, plot style. Currently available are
      `wyoming`, `dwd_a0` and `dwd_a4`. Defaults to `dwd_a0`.
    :type style: str
    :param title: optional, plot title.
    :type style: str
    :param fname: optional, filename (optionally including path)
      to save the plot in.
      If not present, the plot is shown on the screen.
    :type fname: str or path-like object
    :param fmt: optional,
        the file format, e.g. 'png', 'pdf', 'svg',
        If missing, the file format is nfered from the filename extension
        of `fname`.
    :param dpi: optional, plot resolution in dots per inch.
        Defaults to 300.
    """
    if not have_matplotlib:
        raise EnvironmentError('matplotlib not available')
    if fname is None and not have_display:
        raise EnvironmentError('no display, cannot show chart')

    def p2z(p: float) -> float:
        r"""
        convert pressure to Stüve vertical coordinate :math:`p^\kappa`
        :param p: pressure in hPa
        :type p: float
        :return: vertical coordinate (unitless)
        :rtype:  float
        """
        return p ** 0.2859

    def h2z(h):
        r"""
        convert altitude to Stüve vertical coordinate :math:`p^\kappa`
        :param h: pressure in hPa
        :type h: float
        :return: vertical coordinate (unitless)
        :rtype:  float
        """
        return p2z(p_iso(h, hPa=True))

    #
    # check arguments
    #

    if style is None:
        style = 'dwd_a0'
    if style == 'dwd_a0':
        sty = {
            "figsize": (32, 40),
            "trange": (-80, 50),
            "prange": (1050, 1),
            "meters": [
                {"at": np.arange(0., 100000., 1000.),
                 "unit": "km",
                 }
            ],
            "isotherms": [
                {"at": list(np.arange(-80, 50, 10)),
                 "color": "green",
                 "width": "1.5",
                 "style": "-"
                 },
                {"at": list(np.arange(-80, 50, 1)),
                 "color": "green",
                 "width": "0.5",
                 "style": "-"
                 }

            ],
            "isobars": [
                {"at": (list(np.arange(1050, 250, -50)) +
                        list(np.arange(250, 100, -25)) +
                        list(np.arange(100, 20, -10)) +
                        list(np.arange(20, 10, -5)) +
                        list(np.arange(10, 1.1, -2.5))
                        ),
                 "color": "green",
                 "width": "0.5",
                 "style": "-"
                 },
                {"at": (list(np.arange(1050, 250, -10)) +
                        list(np.arange(250, 100, -5)) +
                        list(np.arange(100, 20, -2)) +
                        list(np.arange(20, 10, -1)) +
                        list(np.arange(10, 1.1, -0.5))
                        ),
                 "color": "green",
                 "width": "1.5",
                 "style": "-"
                 },
            ],
            "dryadiabates": [
                {"at": list(np.arange(-70, 120, 10)),
                 "color": "green",
                 "width": "1",
                 "style": "-"
                 }
            ],
            "moistdiabates": [
                {"at": list(np.arange(-70, 120, 10)),
                 "label": "the",
                 "color": "red",
                 "width": "1",
                 "style": "-"
                 }
            ],
            "mixingratios": [
                {"at": (list(np.arange(0.2, 1.1, 0.1)) +
                        [1.2, 1.6] +
                        list(np.arange(2.0, 5.0, 0.5)) +
                        list(np.arange(5., 20., 1.)) +
                        list(np.arange(20., 30., 4.)) +
                        list(np.arange(30., 45., 5.))),  # g/kg
                 "color": "red",
                 "width": "1",
                 "style": "-"
                 }
            ],
            "temperature": {
                "color": "black",
                "width": "2",
                "style": "-"
            },
            "dewpoint": {
                "color": "blue",
                "width": "2",
                "style": "-"
            },
        }
    elif style == 'dwd_a4':
        sty = {
            "figsize": (8, 11),
            "trange": (-60, 40),
            "prange": (1050, 200),
            "meters": [
                {"at": np.arange(0., 12000., 1000.),
                 "unit": "km",
                 }
            ],
            "isotherms": [
                {"at": list(np.arange(-50, 40, 10)),
                 "color": "green",
                 "width": "1.",
                 "style": "-"
                 },
                {"at": list(np.arange(-55, 45, 1)),
                 "color": "green",
                 "width": "0.33",
                 "style": "-"
                 }

            ],
            "isobars": [
                {"at": (list(np.arange(1050, 250, -50)) +
                        list(np.arange(250, 100, -25)) +
                        list(np.arange(100, 20, -10)) +
                        list(np.arange(20, 10, -5)) +
                        list(np.arange(10, 1.1, -2.5))
                        ),
                 "color": "green",
                 "width": "1.",
                 "style": "-"
                 },
                {"at": (list(np.arange(1050, 250, -10)) +
                        list(np.arange(250, 100, -5)) +
                        list(np.arange(100, 20, -2)) +
                        list(np.arange(20, 10, -1)) +
                        list(np.arange(10, 1.1, -0.5))
                        ),
                 "color": "green",
                 "width": "0.33",
                 "style": "-"
                 },
            ],
            "dryadiabates": [
                {"at": list(np.arange(-50, 120, 10)),
                 "color": "green",
                 "width": "0.5",
                 "style": "-"
                 }
            ],
            "moistdiabates": [
                {"at": list(np.arange(-70, 120, 10)),
                 "label": "the",
                 "color": "red",
                 "width": "0.5",
                 "style": "-"
                 }
            ],
            "mixingratios": [
                {"at": (list(np.arange(0.2, 1.1, 0.1)) +
                        [1.2, 1.6] +
                        list(np.arange(2.0, 5.0, 0.5)) +
                        list(np.arange(5., 20., 1.)) +
                        list(np.arange(20., 30., 4.)) +
                        list(np.arange(30., 45., 5.))),  # g/kg
                 "color": "red",
                 "width": "0.5",
                 "style": "--"
                 }
            ],
            "temperature": {
                "color": "black",
                "width": "1.5",
                "style": "-"
            },
            "dewpoint": {
                "color": "blue",
                "width": "1.5",
                "style": "-"
            },
        }
    elif style == 'wyoming':
        sty = {
            "figsize": (6, 6),
            "trange": (-80, 45),
            "prange": (1050, 100),
            "meters": [
                {"at": [altitude(p) for p in np.arange(1000., 100., -100.)],
                 "unit": "km",
                 }
            ],
            "isotherms": [
                {"at": list(np.arange(-80, 41, 10)),
                 "color": "blue",
                 "width": "0.5",
                 "style": "-"
                 },
                {"at": list(np.arange(-75, 46, 10)),
                 "color": "blue",
                 "width": "0.5",
                 "style": "-"
                 },
            ],
            "isobars": [
                {"at": list(np.arange(1000, 99, -100)),
                 "color": "blue",
                 "width": "0.5",
                 "style": "-"
                 },
                {"at": list(np.arange(1050, 149, -100)),
                 "color": "blue",
                 "width": "0.5",
                 "style": "-"
                 },
            ],
            "dryadiabates": [
                {"at": list(np.arange(-60, 181, 20)),
                 "color": "green",
                 "width": "0.5",
                 "style": "-"
                 }
            ],
            "moistdiabates": [
                {"at": list(np.arange(-50, 131, 20)),
                 "label": "the",
                 "color": "blue",
                 "width": "0.5",
                 "style": "-"
                 }
            ],
            "mixingratios": [
                {"at": [0.1, 0.4, 1, 2, 4, 7, 10, 16, 24, 32, 40],  # g/kg
                 "color": "purple",
                 "width": "0.5",
                 "style": "-"
                 }
            ],
            "temperature": {
                "color": "black",
                "width": "1.5",
                "style": "-"
            },
            "dewpoint": {
                "color": "black",
                "width": "1.5",
                "style": "-"
            },
        }
    else:
        raise ValueError(f'Stueve style unknown: {style}')

    # plot boundaries
    tmin, tmax = sty["trange"]
    pmax, pmin = sty["prange"]
    zmin = p2z(pmax)
    zmax = p2z(pmin)

    # start plot
    fig, ax1 = plt.subplots(figsize=sty["figsize"],
                            dpi=dpi)
    plt.subplots_adjust(left=0.15)
    ax1.set_xlim(tmin, tmax)
    ax1.set_ylim(zmin, zmax)
    ax1.set_xticks([x for x in sty["isotherms"][0]["at"]
                    if tmin <= x <= tmax])
    ax1.set_yticks([p2z(x) for x in sty["isobars"][0]["at"]
                    if pmin <= x <= pmax],
                   ["%4.0f" % x for x in sty["isobars"][0]["at"]
                    if pmin <= x <= pmax])
    meter = [x for x in sty["meters"][0]["at"]
             if pmin <= p_iso(x, hPa=True) <= pmax]
    munit = sty["meters"][0]["unit"]
    if munit == "m":
        mstr = "m"
        mfac = 1.
    elif munit == "km":
        mstr = "km"
        mfac = 1. / 1000.
    elif munit == "kft":
        mstr = "kft"
        mfac = 3.28084 / 1000.
    else:
        mstr = ""
        mfac = 1.
    ax2 = ax1.twinx()
    ax2.set_ylim(zmin, zmax)
    ax2.set_yticks([p2z(p_iso(x, hPa=True)) for x in meter],
                   ["%3.0f%s" % (x * mfac, mstr) for x in meter])

    # add isolines
    for isotherms in sty["isotherms"]:
        for x in isotherms["at"]:
            ax1.axvline(x=x,
                        color=isotherms["color"],
                        linestyle=isotherms["style"],
                        linewidth=isotherms["width"])
    for isobars in sty["isobars"]:
        for x in isobars["at"]:
            ax1.axhline(y=p2z(x),
                        color=isobars["color"],
                        linestyle=isobars["style"],
                        linewidth=isobars["width"])
    for dryads in sty["dryadiabates"]:
        for x in dryads["at"]:
            pp, tt = dryad(x, pmax, pmin)
            ax1.plot(tt, [p2z(x) for x in pp],
                     color=dryads["color"],
                     linestyle=dryads["style"],
                     linewidth=dryads["width"])
    for satads in sty["moistdiabates"]:
        for x in satads["at"]:
            pp, tt = satad(x, pmax, pmin,
                           label=satads["label"])
            ax1.plot(tt, [p2z(x) for x in pp],
                     color=satads["color"],
                     linestyle=satads["style"],
                     linewidth=satads["width"])
    for ms in sty["mixingratios"]:
        for x in ms["at"]:
            pp, tt = tdm(x, pmax,  pmin)
            ax1.plot(tt, [p2z(x) for x in pp],
                     color=ms["color"],
                     linestyle=ms["style"],
                     linewidth=ms["width"])

    # add sounding if supplied
    if p is not None and t is not None:
        ax1.plot(t, [p2z(x) for x in p],
                 color=sty["temperature"]["color"],
                 linestyle=sty["temperature"]["style"],
                 linewidth=sty["temperature"]["width"])
    if p is not None and td is not None:
        ax1.plot(td, [p2z(x) for x in p],
                 color=sty["dewpoint"]["color"],
                 linestyle=sty["dewpoint"]["style"],
                 linewidth=sty["dewpoint"]["width"])

    if title is not None:
        plt.title(label=title)
    if fname:
        plt.savefig(fname, format=fmt, dpi=dpi)
    else:
        plt.show()


# ----------------------------------------------------------------------
# plot archived radiosonde
# ----------------------------------------------------------------------


def plot_radisonde(data, diagram=None, **kwargs):
    """
    Plot a radiosonde sounding contained in `data`

    :param data: Sounding data. Either a pandas DataFrame or a dict
      of array-like objects. If data is a dict, all array-like objects
      must be one-dmensional and of the same lenght.
      `data` must contain columns / elements for
      pressure, temperature and humidity.
      Humidity can be specified as dew point, relative humidity or
      dewpoint depression.
    :type data: `pandas.DataFrame` or `dict`
    :param diagram: type of diagram to draw.
      Currently only 'stueve' is available.
    :type diagram: str
    :param kwargs: arguments passed to the diagram function


      | Accepted names for pressure are:
        'p', 'pp', 'P', 'PRES', 'PRESS', 'pressure'
      | Accepted names for temperature are:
        't', 'tt', 'T', 'TEMP', 'temperature'
      | Accepted names for dewpoint are:
        'td', 'TD', 'DWPT', 'dewpoint'
      | Accepted names for dewpoint depression are:
        'dtd', 'DTD', 'DPDP', 'spread'
      | Accepted names for relative humidity are:
        'rh', 'RH', 'RELH', 'relative humidity'

    """
    def _getcol(dataobject: dict | pd.DataFrame, names):
        """

        :type dataobject: object
        """
        for col in names:
            if col in dataobject.columns:
                res = dataobject[col]
                break
        else:
            res = None
        return res

    p_names = ['p', 'pp', 'P', 'PRES', 'PRESS', 'pressure']
    t_names = ['t', 'tt', 'T', 'TEMP', 'temperature']
    td_names = ['td', 'TD', 'DWPT', 'dewpoint']
    sp_names = ['dtd', 'DTD', 'DPDP', 'spread']
    rh_names = ['rh', 'RH', 'RELH', 'relative humidity']

    style = kwargs.pop('style', {})
    if diagram is None:
        diagram = 'stueve'
        style = 'dwd_a4'

    if isinstance(data, (pd.DataFrame, dict)):
        p = _getcol(data, p_names)
        t = _getcol(data, t_names)
        td = _getcol(data, td_names)
        sp = _getcol(data, sp_names)
        rh = _getcol(data, rh_names)
    else:
        raise ValueError('data is fo unsopported type')

    if td is None:
        if sp is not None:
            td = [xt - xsp for xt, xsp in zip(t, sp)]
        elif rh is not None:
            for xt, xp, xrh in zip(t, p, rh):
                td = Humidity(t=xt, p=xp, rh=xrh).td(Kelvin=False)
        else:
            logger.warning('no humidty info')
            td = [np.nan for x in t]

    if np.nanmax(p) > 1500.:
        t = [x/100. for x in p]    # -> hPa
    if np.nanmax(t) > 100.:
        t = [KtoC(x) for x in t]   # -> C
    if np.nanmax(td) > 100.:
        td = [KtoC(x) for x in td]  # -> C

    if diagram == 'stueve':
        stueve(p, t, td, style=style, **kwargs)
